# Copyright (C) 2011-2014 Jens Kasten. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
# 

"""
Package for RSBAC.

Copyright (C) Jens Kasten. All Rights Reserved.
"""

__author__  = "Jens Kasten <jens@kasten-edv.de>"
__status__  = "beta"
__date__    = "10 July 2014"

__all__ = ["which", "get_pid_from_name", "get_linux_dist_distrubution"
    "class_for_name"]


import os
import sys
from subprocess import Popen, PIPE
import platform
import importlib

import logging
logging.basicConfig(format='%(levelname)s:%(name)s:line %(lineno)s: %(message)s')
log = logging.getLogger(__name__)


RSBAC_PROC_INFO_DIR = "/proc/rsbac-info"

ADMIN_PATHS = ["/bin", "/usr/bin", "/sbin", "/usr/sbin", "/usr/local/bin",
    "/usr/local/sbin", "/usr/lib", "/usr/libexec"]

def which(file_name):
    """Python implementation of which. Filter out /usr/local/jails."""
    if os.path.isdir(file_name):
        return file_name
    for path in ADMIN_PATHS:
        service = os.path.join(path, file_name)
        if os.path.isfile(service):
            return service
    for path in os.environ["PATH"].split(":"):
        # exclude the /usr/local/jails path to avoid endless loop
        # this could be happend when the /usr/local/jails on first place
        # in the PATH environment
        if path == "/usr/local/jails":
            continue
        service = os.path.join(path, file_name)
        if os.path.isfile(service):
            return service
    log.info("Could not found %s." % file_name)
    return False

def get_version():
    """Return version number."""
    import rsbactools.__version__
    return __version__.__version__

def get_pid_from_name(prog_name):
    """Try to get the pid number from a give program name.
    If more then one pids are delivered its return the first one.
    On success return int otherwise None."""
    process = Popen([which("pgrep"), prog_name], stdout=PIPE, stderr=PIPE)
    output, error = process.communicate()
    if process.returncode != 0:
        log.info(error.decode())
        return False
    else:
        output = output.decode().split("\n")
        try:
            return int(output[0])
        except IndexError as error:
            log.info("Not found pid info for %s:" % prog_name)
            return False

def get_linux_distribution(full=False):
    """Return the name of linux distribution.
    If full not set return the name as lower string.
    Otherwise a tuple(name, version)."""
    try:
        result = platform.linux_distribution()
        if not full:
            return result[0].split(" ")[0].lower()
        else:
            return (result[0], result[1])
    except IndexError as error:
        log.info(error)


def class_for_name(module_name, class_name):
    # load the module, will raise ImportError if module cannot be loaded
    m = importlib.import_module(module_name)
    # get the class, will raise AttributeError if class cannot be found
    c = getattr(m, class_name)
    return c




# Copyright (C) 2011-2016 Jens Kasten. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
# 

"""
Jail package for RSBAC JAIL module.

Copyright (C) Jens Kasten. All Rights Reserved.
"""

__author__  = "Jens Kasten <jens@kasten-edv.de>"
__status__  = "beta"
__version__ = "0.5"
__date__    = "08 July 2014"

__all__ = ["RunJail", ]


import os
import sys

import logging
logging.basicConfig(format='%(levelname)s:%(name)s:line %(lineno)s: %(message)s')
log = logging.getLogger(__name__)

try:
    from rsbactools import config, converter, get_pid_from_name, rsbac, which
except ImportError as error:
    raise RuntimeError (error)


# Path where jail config files located
RSBAC_JAIL_CONFIG_DIR = "/etc/rsbac/jail" 

# list of all available jail attribute
RSBAC_JAIL_ATTRIBUTE = [
    'jail_id',
    'jail_ip',
    'jail_flags',
    'jail_max_caps',
    'jail_scd_get',
    'jail_scd_modify'
]

# coresponde with the jail config keys
# { "jail-flags": ["allow-dev-read"] }
# call in rsbactools/jail/helper.py the class JailFlags
RSBAC_JAIL_CLASS_MAPPER = {
    "jail-flags": "JailFlags",
    "max-caps": "JailMaxCaps",
    "scd-get": "JailScdGet",
    "scd-read": "JailScdGet",
    "scd-modify": "JailScdModify",
    "scd-set": "JailScdModify",
    "jail-ip": "JailIp",
    "chroot": "JailChroot",
}

# jail flags value are taken from 
# rsbac/types.h line around 462
RSBAC_JAIL_FLAGS = {
    "allow-external-ipc": (1, "-i"),
    "allow-all-net-family": (2, "-n"),
    "allow-inet-raw": (8, "-r"),
    "auto-adjust-ip-address": (16, "-a"),
    "allow-inet-localhost": (32, "-o"),
    "allow-dev-get-status": (128, "-e"),
    "allow-dev-mod-system": (256, "-E"),
    "allow-dev-read": (512, "-d"),
    "allow-dev-write": (1024, "-D"),
    "allow-tty-open": (2048, "-t"),
    "allow-ipc-parent": (4096, "-P"),
    "allow-suid": (8192, "-s"),
    "allow-mount": (16384, "-u"),
    "this-is-syslog": (32768, "-Y"),
    "allow-ipc-syslog": (65536, "-y"),
    "allow-netlink": (131072, "-K"),
    "allow-netlink-mod-system": (262142, "-k"),
    "virtual-user": ("virtual-user", "-V"),
    "private-namespace": ("private-namsespace", "-N"),
}
       
RSBAC_JAIL_MAX_CAPS = {
    "chown": (0, "CHOWN"),
    "dac-override": (1, "DAC_OVERRIDE"),
    "dac-read-search": (2, "DAC_READ_SEARCH"),
    "fowner": (3, "FOWNER"),
    "fsetid": (4, "FSETID"),
    "kill": (5, "KILL"),
    "setgid": (6, "SETGID"),
    "setuid": (7, "SETUID"),
    "setpcap": (8, "SETPCAP"),
    "linux-immutable": (9, "LINUX_IMMUTABLE"),
    "net-bind-service": (10, "NET_BIND_SERVICE"),
    "net-broadcast": (11, "NET_BROADCAST"),
    "net-admin": (12, "NET_ADMIN"),
    "net-raw": (13, "NET_RAW"),
    "ipc-lock": (14, "IPC_LOCK"),
    "ipc-owner": (15, "IPC_OWNER"),
    "sys-module": (16, "SYS_MODULE"),
    "sys-rawio": (17, "SYS_RAWIO"),
    "sys-chroot": (18, "SYS_CHROOT"),
    "sys-ptrace": (19, "SYS_PTRACE"),
    "sys-pacct": (20, "SYS_PACCT"),
    "sys-admin": (21, "SYS_ADMIN"),
    "sys-boot": (22, "SYS_BOOT"),
    "sys-nice": (23, "SYS_NICE"),
    "sys-resource": (24, "SYS_RESOURCE"),
    "sys-time": (25, "SYS_TIME"),
    "sys-tty-config": (26, "SYS_TTY_CONFIG"),
    "mknod": (27, "MKNOD"),
    "lease": (28, "LEASE"),
    "audit-control": (30, "AUDIT_CONTROL"),
    "audit-write": (29, "AUDIT_WRITE"),
    "setfcap": (31, "SETFCAP"),
}

# cap flags are taken from 
# linux/capabilities.h
# FIXME: get source for int values
RSBAC_JAIL_SCD = {
    "time-strucs": (0, "time_strucs"),
    "clock": (1, "clock"),
    "host-id": (2, "host_id"),
    "net-id": (3, "net_id"),
    "ioports": (4, "ioports"),
    "rlimit": (5, "rlimit"),
    "swap": (6, "swap"),
    "syslog": (7, "syslog"),
    "rsbac": (8, "rsbac"),
    "rsbac-log": (9, "rsbac_log"),
    "other": (10, "other"),
    "kmem": (11, "kmem"),
    "network": (12, "network"),
    "firewall": (13, "firewall"),
    "priority": (14, "priority"),
    "sysfs": (15, "sysfs"),
    "rsbac-remote-log": (16, "rsbac_remote_log"),
    "quota": (17, "quota"),
    "sysctl": (18, "sysctl"),
    "nfsd": (19, "nfsd"),
    "ksyms": (20, "ksyms"),
    "mlock": (21, "mlock"),
    "capability": (22, "capability"),
    "kexec": (23, "kexec"),
    "videomem": (24, "videomem"),
}


class JailFlags(object):
    
    def __init__(self):
        self.params = []

    def get(self, params, config_file):
        for i in params:
            try:
                self.params.append(RSBAC_JAIL_FLAGS[i][1])
            except KeyError as error:
                log.error(error)
                sys.stderr.write("Not valid value in '%s' for 'jail-flags'\n" % config_file)
                sys.stderr.write("Available jail-flags:\n")
                for i in sorted(RSBAC_JAIL_FLAGS):
                    sys.stderr.write("%-21s %-21s\n" % ("", i))
                sys.exit(21)
        return self.params


class JailChroot(object):
    
    def __init__(self):
        self.params = ["-R"]

    def get(self, params, config_file):
        self.params.append(params)
        return self.params


class JailIp(object):
    
    def __init__(self):
        self.params = ["-I"]

    def get(self, params, config_file):
        self.params.append(params)
        return self.params


class JailMaxCaps(object):
    
    def __init__(self):
        self.params = ["-C"]

    def get(self, params, config_file):
        for i in params:
            try:
                self.params.append(RSBAC_JAIL_MAX_CAPS[i][1])
            except KeyError as error:
                log.error(error)
                sys.stderr.write("Not valid value in '%s' for 'max-caps\n'" % config_file)
                sys.stderr.write("Available max-caps:\n")
                for i in sorted(RSBAC_JAIL_MAX_CAPS):
                    sys.stderr.write("%-21s %-21s \n" % ("", i))
                sys.exit(22)
        return self.params


class JailScdGet(object):
    
    def __init__(self):
        self.params = ["-G"]

    def get(self, params, config_file):
        for i in params:
            try:
                self.params.append(RSBAC_JAIL_SCD[i][1])
            except KeyError as error:
                log.error(error)
                sys.stderr.write("Not valid value in '%s' for 'scd-get'\n" % config_file)
                sys.stderr.write("Available scd-get:\n")
                for i in sorted(RSBAC_JAIL_SCD):
                    sys.stderr.write("%-21s %-21s\n" % ("", i))
                sys.exit(23)
        return self.params


class JailScdModify(object):
    
    def __init__(self):
        self.params = ["-M"]

    def get(self, params, config_file):
        for i in params:
            try:
                self.params.append(RSBAC_JAIL_SCD[i][1])
            except KeyError as error:
                log.error(error)
                sys.stderr.write("Not valid value in '%s' for 'scd-modify'\n" % config_file)
                sys.stderr.write("Available scd-modify:\n")
                for i in sorted(RSBAC_JAIL_SCD):
                    sys.stderr.write("%-21s %-21s\n" % ("", i))
                sys.exit(24)
        return self.params


class JailParams(object):

    def __init__(self):
        self.params = []

    def set_log_level(self, log_level):
        log.setLevel(log_level)

    def build(self, policies, config_file):
        for key in policies.keys():
            try:
                log.debug("call: %s()" % RSBAC_JAIL_CLASS_MAPPER[key])
                obj = getattr(sys.modules[__name__], RSBAC_JAIL_CLASS_MAPPER[key])()
                log.debug("set: %s" % policies[key])
                params = obj.get(policies[key], config_file)
                log.debug("get: %s" % params)
                self.params += params
            except KeyError as error:
                log.error(error)
                sys.stderr.writer("Could not found ClassName: %s" % key)
                sys.exit(20)
        return True
       
    def get(self):
        return self.params


class RunJail(object):
    """Tool to configure a rsbac_jail through a json configuration file."""

    def __init__(self, args):
        self.args = args
        self.policies = {}
        self.params = []
       
        if "dry_run" in args and args["dry_run"]:
            self.set_log_level(logging.INFO)
        if "verbose" in args and args["verbose"]:
            self.set_log_level(logging.DEBUG)
    
    def set_log_level(self, log_level):
        log.setLevel(log_level)

    def get_log_level(self):
        return log.getEffectiveLevel()

    def set_policies(self, config_file):
        log.debug("loading config: %s" % config_file)
        jl = config.JsonLoader()
        jl.set_log_level(self.get_log_level())
        if jl.add_config(config_file) and jl.load_config():
            self.policies = jl.get_values()
            log.debug("loading values: %s" % self.policies)
            return True

    def get_policies(self):
        return self.policies
    
    def set_params(self, config_file):
        p = JailParams()
        p.set_log_level(self.get_log_level())
        if p.build(self.get_policies(), config_file):
            self.params = p.get()
            log.debug("converted values: %s" % self.params)
            return True

    def get_params(self):
        return self.params 

    def prepare(self):
       if self.args.get("config_file", False):
            config_file = os.path.join(RSBAC_JAIL_CONFIG_DIR, 
                                       ".".join([self.args["config_file"], 
                                                "json"]))
            if not self.set_policies(config_file):
                return False
            if not self.set_params(config_file):
                return False
            return True

    def run(self):
        self.prepare()
     
        if not self.args.get("program", []):
            log.error("Need something to put into a jail.")
            return False
      
        rsbac_jail = which("rsbac_jail")
        program = which(self.args.get("program", False)[0])
        program_params = self.args.get("program", "")[1:] 

        if rsbac.Rsbac().get_module("JAIL") == "on":
            log.debug("running in jail mode")
            jail = True
        else:
            log.debug("not running in jail mode")
            jail = False
       
        args = []
        if jail:
            prog = rsbac_jail
            args.append(prog)
            args.extend(self.get_params())
            args.append(program)
            args.extend(program_params)
        else:
            prog = program
            args.append(prog)
            args.extend(program_params)

        if self.args.get("show"):
            output = rsbac_jail + " " + " ".join([i for i in self.get_params()])
            output += " " + program +  " " + " ".join([str(i) for i in program_params])
            print(output)

        elif self.args.get("dry_run"):
            log.info("args: %s" % self.args)
            log.info("this would execute through os.excev()")
            log.info("%s %s" % (prog, args))
        else:
            try:
                os.execv(prog, args)
            except OSError as error:
                log.error(error)


class PsJail(object):
    """Class to display process jail information."""

    def __init__(self):
        self.processes = {}
        self.processes_info = {}
        self.converter = converter.Converter()

    def list_process(self, prog_name):
        """Search for pid and add them to dictionary."""
        pid = get_pid_from_name(prog_name)
        if pid:
            self.processes[str(pid)] = prog_name
            return True
        else:
            return False

    def list_processes(self):
        """Search for pid in proc directory."""
        for pid in os.listdir("/proc"):
            if pid.isdigit():
                status = os.path.join("/proc", pid, "status")
                with open(status, "r") as fd:   
                    try:
                        name = fd.readline()
                        # add pid as key and processname as value for dict processes
                        self.processes[pid] = name[6:].strip() 
                    except PermissionError as error:
                        pass
                    except OSError as error:
                        pass

    def get_processes_info(self, jail_id_only=False):
        """Collect all available information from rsbac jail."""
        rsbac_api = rsbac.Rsbac()

        # collect all jail infos 
        for pid, process_name in self.processes.items():
            # first get jail_id to test if further action is needed
            jail_id, error = rsbac_api.attr_get_process("JAIL", pid, "jail_id")
            if error:
                continue
            # check if jail_id greater then zero, that process is in jail
            jail_id = jail_id.strip()
            if int(jail_id) == 0:
                continue
            if jail_id_only and jail_id_only != int(jail_id):
                continue
            # assign values to dict    
            self.processes_info[process_name] = {
                "jail_id": jail_id,
                "pid": pid,
            }
          
            for attribute in RSBAC_JAIL_ATTRIBUTE:
                # dont query for jail_id again because already get
                if attribute == "jail_id":
                    continue
                attr, errors = rsbac_api.attr_get_process("JAIL", pid, attribute)
                if error:
                    continue
                if attribute == "jail_ip":
                    self.processes_info[process_name][attribute] = attr.strip()
                elif int(attr) > 0:
                    # get human readable names
                    names = self.get_names_from_number(attribute, attr)
                    self.processes_info[process_name][attribute] = names

    def get_names_from_number(self, attribute, number):
        """Return a list of human readable names."""
        names = []
        if attribute == "jail_flags":
            numbers = self.converter.int_to_bin(number, 32)
            for name, key in RSBAC_JAIL_FLAGS.items():
                if key[0] in numbers:
                    names.append(name)
            return names
        elif attribute == "jail_max_caps":
            numbers = self.converter.bin_to_int(number)
            for name, key in RSBAC_JAIL_MAX_CAPS.items():
                if key[0] in numbers:
                    names.append(name)
            return names
        else:
            numbers = self.converter.bin_to_int(number)
            for name, key in RSBAC_JAIL_SCD.items():
                if key[0] in numbers:
                    names.append(name)
            return names
    
    def show(self):
        """Show the collection information on a terminal with 80 char wide."""
        header = {}
        header["jail_flags"] = "Jail-Flags"
        header["jail_max_caps"] = "Jail-Max-Caps"
        header["jail_scd_get"] = "Jail-SCD-Get"
        header["jail_scd_modify"] = "Jail-SCD-Modify"

        if len(self.processes_info.items()) == 0:
            return 
        
        for process_name, value in sorted(self.processes_info.items()):
            print("-" * 79)
            print("%-18s  Pid: %-6s  Jail Id: %-6s  Jail Ip: %-16s" % (
                process_name, value["pid"], value["jail_id"], value["jail_ip"]))
            print("-" * 79)

            if "jail_flags" in value:
                jail_flags = sorted(value["jail_flags"])
                first = jail_flags[0]
                del jail_flags[0]
                print("%-15s: %s" % (header["jail_flags"], first))
                for i in jail_flags:
                    print("%-16s %s" % ("", i))

            if "jail_max_caps" in value:
                jail_max_caps = sorted(value["jail_max_caps"])
                first = jail_max_caps[0]
                del jail_max_caps[0]
                print("%-15s: %s" % (header["jail_max_caps"], first))
                for i in jail_max_caps:
                    print("%-16s %s" % ("", i))
            if "jail_scd_get" in value:
                jail_scd_get = sorted(value["jail_scd_get"])
                first = jail_scd_get[0]
                del jail_scd_get[0]
                print("%-15s: %s" % (header["jail_scd_get"], first))
                for i in jail_scd_get:
                    print("%-16s %s" % ("", i))
            if "jail_scd_modify" in value:
                jail_scd_modify = sorted(value["jail_scd_modify"])
                first = jail_scd_modify[0]
                del jail_scd_modify[0]
                print("%-15s: %s" % (header["jail_scd_modify"], first))
                for i in jail_scd_modify:
                    print("%-16s %s" % ("", i))




#
# API for python binding of rsbac
# In then moment just placeholder
#

import os
import sys
from subprocess import Popen, PIPE

try:
    from rsbactools import RSBAC_PROC_INFO_DIR
except ImportError as error:
    print(error)
    sys.exit(-1)


class Rsbac(object):
    """Should later replaced by C implementation."""
    
    def __init__(self, verbose=False):
        self.module_on = []
        self.module_off = []
        self.verbose = verbose

    def is_rsbac(self):
        """Simply check if proc directory has subdirectory rsbac."""
        if os.path.isdir(RSBAC_PROC_INFO_DIR):
            return True

    def execute(self, cmd, *args):
        """Generic methode to call Popen().
        Return the result.
        """
        if isinstance(cmd, str):
            cmd = [cmd]
        for i in args:
            if isinstance(i, list):
                cmd.extend(i)
            elif isinstance(i, str):
                cmd.append(i)
        if self.verbose:
            print(" ".join(cmd))
        try:
            process = Popen(cmd, stdout=PIPE, stderr=PIPE)
            output, error = process.communicate()
            if process.returncode != 0:
                output = output.decode("utf-8").strip()
                return (output, True)
            elif len(error) > 0:
                if error:
                    error = error.decode("utf-8").strip()
                return (error, True)
            else:
                if output:
                    output = output.decode("utf-8").strip()
                return (output, False)
        except IOError as error:
            return (False, "Failure: %s" % error)
  
    def attr_set_fd(self, *args):
        return self.execute("attr_set_fd", *args)
   
    def attr_get_fd(self, *args):
        return self.execute("attr_get_fd", *args)

    def attr_set_user(self, *args):
        return self.execute("attr_set_user", *args)
   
    def attr_get_user(self, *args):
        return self.execute("attr_get_user", *args)

    def attr_set_file_dir(self, *args):
        return self.execute("attr_set_file_dir", *args)
    
    def attr_get_file_dir(self, *args):
        return self.execute("attr_get_file_dir", cmd, *args)

    def rc_set_item(self, *args):
        return self.execute("rc_set_item", *args)

    def rc_get_item(self, *args):
        return self.execute("rc_get_item", *args)

    def rc_copy_type(self, *args):
        return self.execute("rc_copy_type", *args)

    def attr_get_process(self, *args):
        return self.execute("attr_get_process", *args)

    def attr_back_fd(self, *args):
        return self.execute("attr_back_fd", *args)

    def auth_back_cap(self, *args):
        return self.execute("auth_back_cap", *args)
    
    def acl_tlist(self, *args):
        return self.execute("acl_tlist", *args)

    def acl_mask(self, *args):
        return self.execute("acl_mask", *args)

    def get_modules(self, active=False):
        """Retun rsbac module as a dictonary.
        set self.module_on as list with all active module
        set self.module_off as list with all deactive module
        return rsbac_module 
        """
        result = {"Module": {}}
        try:
            with open(os.path.join(RSBAC_PROC_INFO_DIR, "active")) as fd:
                for line in fd.readlines():
                    i = line.split(':')
                    # check if a module is available
                    if i[0] == "Module":
                        # clean string and create a list
                        j = i[1].strip("\n").lstrip(" ").split(" ")
                        # remove empty elements from list
                        j = list(filter(None, j))
                        if active is False:
                            result['Module'][j[0]] = j[1].lower()
                        elif active == "on":
                            if j[1].lower() == "on":
                                result['Module'][j[0]] = j[1].lower()
                        elif active == "off":
                            if j[1].lower() == "off":
                                result['Module'][j[0]] = j[1].lower()
            return result
        except IOError as error:
            return result

    def get_module(self, module):
        """Return status on|off or False."""
        try:
            if self.get_modules():
                return self.get_modules()["Module"][module]
        except KeyError as error:
           return False


if __name__ == "__main__":
    r = Rsbac()
    print(r.get_modules())



# -*- coding: utf-8 -*-
#
# Copyright (C) 2011-2015 Jens Kasten. All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
# 

"""
FF package for RSBAC FF module.

Copyright (C) Jens Kasten. All Rights Reserved.
"""

__author__  = "Jens Kasten <jens@kasten-edv.de>"
__status__  = "beta"
__date__    = "08 July 2014"

__all__ = ["Converter", "Paths", "PathsFilter", "PathsExclude", "PathsShow",
        "Shields"]


import os
import sys

import logging
logging.basicConfig(format='%(levelname)s:%(name)s:line %(lineno)s: %(message)s')
log = logging.getLogger(__name__)

try:
    from rsbactools import config, converter, rsbac
except ImportError as error:
    print(error)
    sys.exit(-1)

# Path where ff config files located
RSBAC_FF_CONFIG_DIR = "/etc/rsbac/ff"
# maps given keywords form ff config to a class
RSBAC_FF_CLASS_MAPPER = {
    "paths": "Paths",
    "paths-exclude": "PathsExclude",
}
# internal values for module FF to display readable values
RSBAC_FF_FLAGS  = {
    "1": "read_only",
    "2": "execute_only",
    "4": "search_only",
    "8": "write_only",
    "16": "secure_delete",
    "32": "no_execute",
    "64": "no_delete_or_rename",
    "128": "add_inherited",
    "256": "append_only",
    "512":  "no_mount",
    "1024": "no_search",
}
RSBAC_FF_ADD_INHERITED = 128
RSBAC_FF_READ_ONLY = 1

# a approximatly counter of policies
POLICY_COUNTER = 0

def update_counter(file_dir_list=False):
    global POLICY_COUNTER
    if file_dir_list:
        POLICY_COUNTER += len(file_dir_list)
    else:
        POLICY_COUNTER += 1

def get_counter():
    return POLICY_COUNTER


"""
RSBAC ff_flags has a special output values.
This Converter prepare the output and add human readable ff_flags values
to the output.
    Expect rsbac output off
        /proc: RÃ¼ckgabewert: 128
        /bin: RÃ¼ckgabewert: 128
"""
def convert(output):
    """Transform the output always to a list."""
    result = {}
    # list comprehension does first split by line 
    # after split by space
    output = [x.split(" ") for x in output.split("\n") if len(x) > 0]
    try:
        # list comprehension return only path and value
        # remove from first value the last sign 
        for x in output:
            path = x[0].split(":")[0]
            ff_value = x[len(x)-1]
            ff_list = converter.Converter().int_to_bin(ff_value, 32)
            ff_names = [(RSBAC_FF_FLAGS[str(x)], x) for x in ff_list]
            result[path] = {
                "ff_flags": ff_value,
                "ff_list": ff_names
            }
        return result
    except IndexError as error:
        log.error("convert: %s" % error)
    except KeyError as error:
        log.error("Value not in RSBAC_FF_FLAGS: %s" % error)


def format_result(result):
    """Print the formated output."""
    output = convert(result)
    if not output:
        return
    for path, value in sorted(output.items()):
        if len(value["ff_list"]) == 1:
            print("%-54s%-20s% +4s" % (path, value["ff_list"][0][0], 
                value["ff_list"][0][1]))
            continue
        print("%-54s%-20s% +4s" % (path, "", value["ff_flags"]))
        for j in range(len(value["ff_list"])):
            print("%-54s%-20s% +4s" % ("", value["ff_list"][j][0],
                value["ff_list"][j][1]))


class Paths(object):
    """Handle the config key paths."""

    def __init__(self, policies):
        """Argument policies must be a dicts with keyword paths."""
        self.policies = policies["paths"]
        self.args = ["FF", "FD", "ff_flags"] 

    def on(self):
        try:
            for file_dir, ff_flags in self.policies.items():
                args = list(self.args +  [str(ff_flags), file_dir])
                result, error = rsbac.Rsbac().attr_set_fd(args)
                if error:
                    log.error(result)
                else:
                    update_counter()
            return True                        
        except KeyError as error:
            log.error(error)
            return False

    def off(self):
        try:
            for file_dir in self.policies.keys():
                args = list(self.args + [str(RSBAC_FF_ADD_INHERITED), file_dir])
                result, error = rsbac.Rsbac().attr_set_fd(args)
                if error:
                    log.error(result)
                else:
                    update_counter()
            return True
        except KeyError as error:
            log.error(error)
            return False

    def show(self):
        try:
            for file_dir in sorted(self.policies.keys()):
                args = list(self.args + [file_dir])
                result, error = rsbac.Rsbac().attr_get_fd(args)
                if error:
                    log.error(result)
                else:
                    update_counter()
                    format_result(result)
            return True
        except KeyError as error:
            log.error(error)
            return False


class PathsFilter(object):
    """Filter out exclude and exculde-startswitht from file_ dir list"""

    def __init__(self, policies):
        self.policies = policies
         
    def get_paths(self):
        """return a list"""
        return self.policies.keys()

    def get_exclude(self, path):
        """return a list"""
        if "exclude" in self.policies[path]:
            if len(self.policies[path]["exclude"]) > 0:
                return self.policies[path]["exclude"]
        return []

    def get_exclude_startswith(self, path):
        """return a list"""
        if "exclude-startswith" in self.policies[path]:
            if len(self.policies[path]["exclude-startswith"]) > 0:
                return self.policies[path]["exclude-startswith"]
        return []

    def filter_exclude(self, list_to_filter, exclude):
        """return a list"""
        return list(set(list_to_filter) - set(exclude))
    
    def filter_exclude_startswith(self, list_to_filter, exclude):   
        result = list_to_filter
        for i in sorted(list_to_filter):
            for j in exclude:
                if i.startswith(j):
                    result.remove(i)
                    break
        return result

    def get_filtered_list(self):
        """return a list"""
        result = []
        for path in self.get_paths():
            try:
                list_to_filter = os.listdir(path)
            except OSError as error:
                log.error(error)
                continue
            if self.get_exclude(path):
                list_to_filter = self.filter_exclude(list_to_filter,
                    self.get_exclude(path))
            if self.get_exclude_startswith(path):
                list_to_filer = self.filter_exclude_startswith(list_to_filter,
                    self.get_exclude_startswith(path))
            result.extend([os.path.join(path, x) for x in list_to_filter])
        return result


class PathsExclude(object):
    """Handle the config key paths-exclude."""

    def __init__(self, policies):
        """Argument policies must be a dicts with keyword paths-exclude."""
        self.policies = policies["paths-exclude"]
        self.args = ["FF", "FD", "ff_flags"] 

    def on(self):
        try:
            file_dir_list = PathsFilter(self.policies).get_filtered_list()
            args = self.args
            args.append(str(RSBAC_FF_READ_ONLY))
            args.extend(file_dir_list)
            result, error = rsbac.Rsbac().attr_set_fd(args)
            if error:
                log.error(result)
            else:
                update_counter(file_dir_list)
            return True                        
        except KeyError as error:
            log.error(error)
            return False

    def off(self):
        try:
            file_dir_list = PathsFilter(self.policies).get_filtered_list()
            args = self.args
            args.append(str(RSBAC_FF_ADD_INHERITED))
            args.extend(file_dir_list)
            result, error = rsbac.Rsbac().attr_set_fd(args)
            if error:
                log.error(result)
            else:
                update_counter(file_dir_list)
            return True
        except KeyError as error:
            log.error(error)
            return False

    def show(self):
        try:
            file_dir_list = PathsFilter(self.policies).get_filtered_list()
            args = self.args
            args.extend(file_dir_list)
            result, error = rsbac.Rsbac().attr_get_fd(args)
            if error:
                log.error(result)
            else:
                update_counter(file_dir_list)
                format_result(result)
            return True
        except KeyError as error:
            log.error(error)
            return False


class Shields(object):

    def __init__(self):
        self.args = {}
        self.policies = {}

    def add_args(self, args):
        for key, value in args.items():
            self.args[key.replace("_", "-")] = value

    def set_log_level(self, log_level):
        log.setLevel(log_level)

    def get_log_level(self):
        return log.getEffectiveLevel()

    def update_policies(self, config_file):
        """"""
        log.debug("loading config: %s" % config_file)
        jl = config.JsonLoader()
        jl.set_log_level(self.get_log_level())
        if jl.add_config(config_file) and jl.load_config():
            log.debug("add policies values")
            self.policies.update(jl.get_values())
            return True
        else:
            return False

    def get_policies(self, key=False):
        try:
            if key and key.startswith("@"):
                return {}
            elif key:
                log.debug("Found key in policies: %s" % key)
                return self.policies[key]
            else:
                return self.policies
        except KeyError as error:
            log.debug("Skip key is not in policies: %s" % key)
            return False

    def load_configs(self, config_dir):
        try:
            for config_file in sorted(os.listdir(config_dir)):
                if config_file.endswith(".json"):
                    self.update_policies(os.path.join(config_dir, config_file))
            return True
        except OSError as error:
            log.warning(error)
            return False
   
    def run(self):
        # set all keys to status off full
        if self.args["full"]:
            for key in self.get_policies().keys():
                self.args[key] = self.args["full"]

        for key in self.args.keys():
            if not self.get_policies(key):
                continue
            if not self.args[key]:
                continue
            for i in self.get_policies(key).keys():
                try:
                    log.debug("call: %s()" % RSBAC_FF_CLASS_MAPPER[i])
                    obj = getattr(sys.modules[__name__], 
                        RSBAC_FF_CLASS_MAPPER[i])(self.get_policies(key))
                except KeyError as error:
                    log.debug(error)
                    log.debug("No class mapper found for: %s" % i)
                    continue
                try:
                    log.debug("call: %s()" % self.args[key])
                    getattr(obj, self.args[key])()
                except TypeError as error:
                    log.debug("failed: %s()" % self.args[key])
                    log.debug(error)
                    return False
        return True

    def get_policies_counter(self):
        return get_counter() 
    



import os
import sys
import re
import json
from configparser import ConfigParser


import logging
logging.basicConfig(format='%(levelname)s:%(name)s: %(message)s')
log = logging.getLogger(__name__)


try:
    from rsbactools import which
except ImportError as error:
    print(error)
    sys.exit(-1)


class JsonLoader(object):
    """Class to load a json file with error handling."""

    def __init__(self):
        self.config_values = None
        self.config_file = None

    def set_log_level(self, log_level):
        log.setLevel(log_level) 

    def add_config(self, config_file):
        if os.path.isfile(config_file):
            self.config_file = config_file
            return True
        else:
            log.warning("FileNotFound: %s" % config_file)
            return False

    def load_config(self):
        if not self.config_file:
            return False
        try:
            with open(self.config_file) as fd:
                self.config_values = json.loads(fd.read())
            return True
        except ValueError as error:
            log.warning("%s in %s" % (error, self.config_file))
            return False

    def get_values(self):
        return self.config_values


class IniLoader(object):
    """ini parser callback"""

    def __init__(self, config_name):
        self.config_name = config_name

    def parse_config(self):
        """parser the config"""
        try:
            cp = ConfigParser.read(self.config_file)
            values = cp.__dict__['_sections'].copy()
            return values
        except Exception as error:
            log.info(error)


class JailParser(object):
    """ 
    Read the jailconfig from adamantix style which are placed 
    in /etc/rsbac/jail and return a dictonary with six parts.
    (c) 2008-2011 Jens Kasten <igraltist@rsbac.org>
    """

    def __init__(self, config_path="/etc/rsbac/jail"):
        # path to jail configuration files
        self.jail_config_path = config_path
        # dictionary to keep jail information
        # it has the same layout like a jail configuration file 
        # the values for jail-flags, max-caps, scd-read, scd-modify
        # would be overriden if values found in configuration file 
        # through a list
        self.jail_parts = {
            'jail-chroot': '',
            'jail-ip': '',
            'jail-flags': 0, 
            'max-caps': 1,
            'scd-read': 2,
            'scd-modify': 3,
        }
        # collect absolute path for binaries
        self.ifconfig = which("ifconfig")
        self.route = which("route")
    
    def check_config_path(self, config):
        """Return the absolute path if jail configuration file exists. """
        config = os.path.join(self.jail_config_path, config)
        if os.path.isfile(config):
            return config 
        else:
            print("Could not found jail configuration file '%s'." % config)
            sys.exit()

    def __call__(self, config):
        """Make it callable."""
        line = []
        start = []
        end = []
        values = []
        
        config = self.check_config_path(config)
        with open(config, "r") as fd:
            for i in fd:
                # remove comments and whitespace
                i = i.replace("\n", "")
                if re.search(';', i):
                    temp_line = i.split(";")[0]
                    if len(temp_line) > 1:
                        line.append(temp_line.strip())
                else: 
                    line.append(i.strip())
        
        # remove empty elements from line list                
        line = [i for i in line if len(i) >= 1]
        for i in range(len(line)):
            if re.search('^"', line[i]) and re.search('"$', line[i]):
                if len(line[i]) == 1:
                    print("This sign \" is not allow as standalone.")
                    print("For comments use sign ;")
                    print("Please edit your configfile, '%s'" % config)
                    sys.exit()
                else:
                    if i == 0:
                        chroot_dir = line[i].replace('"', '')
                        if os.path.isdir(chroot_dir) or len(chroot_dir) == 0:
                            self.jail_parts['jail-chroot'] = line[i].replace('"', '')
                        else:
                            print("Path does not exists %s." % chroot_dir)
                            sys.exit()
                    elif i == 1:
                         ip = line[i].replace('"', '')
                         self.jail_parts['jail-ip'] = self.check_ip(ip, config)
            # this part search for ( and ) on which place they are in list line
            if re.search('\(', line[i]):
                start.append(i)
                line[i] = line[i].replace('(', '').replace(' ', '')
            if re.search('\)', line[i]):
                end.append(i)
                line[i] = line[i].replace(')', '').replace(' ', '')
        if len(start) < len(end):
            print("Missing open '(' in '%s'." % (config))
            sys.exit(1)
        elif len(start) > len(end):    
            print("Missing close ')' in '%s'." % (config))
            sys.exit(1)
        elif len(start) < 4 and len(end) < 4:
            print("Missing on category '()' in '%s'." % config)
            sys.exit(1)
        elif len(start) > 4 and len(end) > 4:
            print("One category '()' is to much in '%s'." % config)
            sys.exit(1)
        # the start and end can only contain 4 elements
        # the values between start and end must be add as categories values
        i = 0
        while i < 4:
            for key, value in self.jail_parts.items():
                if i == value:
                    if start[i] == end[i]:
                        if len(line[start[i]]) == 0:
                            self.jail_parts[key] = line[start[i]]
                        else:
                            # even if is a string, add is as list
                            # for this :start[i]+1
                            self.jail_parts[key] = line[start[i]:start[i]+1]
                    else:
                        self.jail_parts[key] = line[start[i]:end[i]+1]
                        # wehn the list have empty entries remove it
                        for k in self.jail_parts[key]:
                            if len(k) == 0:
                                self.jail_parts[key].remove(k)
            i += 1               
        return self.jail_parts

    def check_ip(self, ip, config):
        """Check ip part."""
        ip_default = "0.0.0.0"
        ip_localhost = "127.0.0.1"
        if len(ip) == 0:
            return ip_default 
        elif ip == "lo" or ip == "localhost":
            return ip_localhost 
        else:
            if self.check_ip_format(ip):
                return ip
            else:
                ip_from_interface = self.check_interface(ip)
                if ip_from_interface:
                    return ip_from_interface
                else:
                    print("Given network interface '%s' is no up in '%s'." % (ip, config))
                    return ip

    def check_ip_format(self, ip):
        """Valided the ip address."""
        ip_pattern = "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."
        ip_pattern += "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."
        ip_pattern += "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."
        ip_pattern += "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"
        if re.compile(ip_pattern).match(ip):
            return True
        else:
            return False

    def check_interface(self, interface):
        ip = []
        cmd = [self.ifconfig, interface]
        try:
            # env must set to obtain english words for searching key
            result = Popen(cmd, stdout=PIPE, stderr=PIPE, env={"LC_ALL":"C"})
            result.wait()
            if result.returncode == 0:
                ip = result.communicate()[0].strip()
                ip = re.findall("inet addr:(\d+\.\d+\.\d+\.\d+)" , ip)
                if len(ip) > 0:
                    return ip[0]
        except OSError as e:
            print(str(e[1]))
        except IOError as e:
            print(str(e[1]))


# Parser to get content from a configfile.
#
# config file syntax:
#       key = value
#       #key = value    <- commented 
#       key = value  # comment
#
"""
(c) 2007-2012 Jens Kasten <jens@kasten-edv.de>
"""

class Parser(object):
    """Simple config parser for key value configuration file."""
   
    def __init__(self, config_name):
        self.config_name = config_name

    def _check_config_syntax(self):
        """Return a cleaned dictionary from a befor readed config file."""
        print(self.config_name)
        if not os.path.isfile(self.config_name):
            return False
        else:
            counter = 1
            config = []
            with open(self.config_name) as fd:
                lines = fd.readlines()
                # remove withespace but not and arguments 
                for line in lines:
                    if len(line) > 1 and not line.startswith('#'):
                        # split only the first equal sign form left side
                        temp = line.strip().split("=", 1)
                        assert len(temp) == 2, "Missing sign '=' in %s on line %s" %  (self.config_name, counter)
                        # remove all withespace from string
                        key = re.sub(r'\s', '', temp[0])
                        assert len(temp) == 2, "Missing value in %s on line %s" % (self.config_name, counter)
                        # remove comment
                        value = temp[1].split("#")[0].strip()
                        content = "=".join([key, value])
                        config.append(content)
                    counter += 1
            return config

    def parse_config(self):
        """Return a dictionary"""
        lines = self._check_config_syntax()
        if not lines:
            return 
        config = {}
        for line in lines:
            if len(line) > 0:
                line = line.split("=", 1)
                config[line[0]] = line[1]
        return config



import codecs
import os
import re

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

# take from pip
def read(*parts):
    # intentionally *not* adding an encoding option to open, See:
    # https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return codecs.open(os.path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


long_description = read("README.rst")


# setup a list to scripts which are run as standalone 
scripts = ["scripts/rsbac-run-jail", "scripts/rsbac-ff-shields",
    "scripts/rsbac-symlink-redirect", "scripts/rsbac-backup",
    "scripts/rsbac-ps-jail"]

setup(
    name="rsbac-tools",
    version=find_version("rsbactools", "__version__.py"),
    long_description=long_description,
    description="Command-line tools for daily tasks on a RSBAC system.",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.4",
        "Topic :: System :: Systems Administration",
    ],
    keywords=["security", "rsbac"],
    author="Jens Kasten",
    author_email="jens@kasten-edv.de",
    url="http://www.kasten-edv.de/projects/rsbac",
    license="GNU GPLv3",
    package_dir={"rsbactools": "rsbactools"},
    packages=["rsbactools"],
    scripts=scripts,
    zip_safe=False
)



#!/usr/bin/env python
#
# Module to get rsbac value from give integer number for human readable.
#
# Depencies:
#           python-argparse
#

"""
(c) 2011 Jens Kasten <jens@kasten-edv.de>
"""

import argparse


class Converter(object):
    """Converter to obtain names from rsbac values."""    
    
    def int_to_bin(self, value, bitmask_len):
        """interger to binary"""
        value = int(value)
        dual_count = 1
        values_to_find = []
        for i in range(bitmask_len):
            bit_count = value >> i & 1  
            if i >= 1:
                dual_count += dual_count
            if bit_count == 1:
                values_to_find.append(dual_count)
            if dual_count >= value:
                return values_to_find

    def show_bits_from_int(self, value, as_dual=False):
        """binary to integer"""
        value = int(value)
        result = []
        bit_count = ""
        while value > 0:
            bit_count = str(value % 2) + bit_count
            value = value >> 1  
        bit_count = "".join(reversed(bit_count))            
        if as_dual:
            for i in range(len(bit_count)):
                if int(bit_count[i]) == 1:
                    result.append(1)
                else:
                    result.append(0)
            result = "".join([str(i) for i in result])
            return result[::-1]
        else:
            for i in range(len(bit_count)):
                if int(bit_count[i]) == 1:
                    result.append(i)
            return result 
        
    def bin_to_int(self, value):
        """binary to integer"""
        value = int(value)
        result = []
        bit_count = ""
        while value > 0:
            bit_count = str(value % 2) + bit_count
            value = value >> 1  
        bit_count = "".join(reversed(bit_count))            
        for i in range(len(bit_count)):
            if int(bit_count[i]) == 1:
                result.append(i)
        return result 


def main():
    parser = argparse.ArgumentParser(description="Converter")
    parser.add_argument("-i", "--int-to-bin", type=int,
        metavar="interger count",
        help="Convert an interger to binary representation")
    parser.add_argument("-I", "--show-bits-from-int", type=int,
        metavar="integer count",
        help="Show the bit representation of an interger. The shown"
            " value are the place where bits are set.")
    parser.add_argument("-b", "--bin-to-int", type=str,
        metavar="dual string",
        help="Convert a binary to interger representation")
    parser.add_argument("-B", "--bitmask-length", default=32,
        action="store_true",
        help="The length of bitmask to using. Default length is 32.")
    parser.add_argument("-S", default=False, action="store_true",
        help="Show result as string")
    
    options = parser.parse_args()
    print(options)
    
    if not options.int_to_bin and not options.show_bits_from_int:
        parser.print_usage()
    c = Converter()

    if options.int_to_bin:
        print(c.int_to_bin(options.int_to_bin, options.bitmask_length))
    if options.show_bits_from_int:
        print(c.show_bits_from_int(options.show_bits_from_int, options.S))
    if options.bin_to_int:
        print(c.bin_to_int(options.bin_to_int))

if __name__ == "__main__":
    main()



#
# function that return the backup commands
# only with prefix backup_ in name        
#

import os
import sys
import re
import argparse
import time
import datetime
from subprocess import Popen, PIPE, call
import shutil
import tempfile
import logging

logging.basicConfig(format='%(levelname)s:%(name)s:line %(lineno)s: %(message)s')
log = logging.getLogger(__name__)

try:
    from rsbactools import rsbac, RSBAC_PROC_INFO_DIR
except ImportError as error:
    sys.exit(error)


# timestamp
TIMESTAMP = str("%0.f" % datetime.datetime.now().timestamp())

# default backup directory
BACKUP_DIRECTORY = os.path.join(os.getcwd(), "backup")
BACKUP_DIRECTORY_TODAY = os.path.join(
     str(datetime.datetime.now().year),
     str(datetime.datetime.now().month),
     str(datetime.datetime.now().day)
)
BACKUP_SHM = "/dev/shm/backup"

# BACKUP_MODULES contains module names which have no entry in 
# /proc/rsbac-info/active or similar
BACKUP_MODULES = ["general", "net", "log"]

# BACKUP_MODULES_EXCLUDE contains module names which are not backup.
#   e.g. JAIL is only process based
BACKUP_MODULES_EXCLUDE = ["JAIL"]

# directories which exclude
DIRECTORIES_EXCLUDE = ["/rsbac.dat", "/proc", "/sys"]

# buffer size for Popen and or open
BUFFER_SIZE = 4096


# command function must return a list 
def command_log():
    cmd = [
        ["switch_adf_log", "-b"]
    ]
    return cmd

def command_net():
    return [
        ["net_temp", "-a", "-b"],
        ["attr_back_net", "-a", "NETDEV"],
        ["attr_back_net", "-a", "NETTEMP"]
    ]

def command_mac():
    return [
        ["mac_back_trusted", "-r", "/"],
        ["attr_back_user", "-a", "-M", "MAC"]
    ]

def command_auth():
    return [
        ["auth_back_cap", "-r", "/"],
        ["attr_back_user", "-a", "-M", "AUTH"]
    ]

def command_cap():
    cmd = [
        ["attr_back_user", "-a", "-M", "CAP"]
    ]
    return cmd

def command_rc():
    return [
        ["rc_get_item", "backup"],
        ["attr", "-a", "-M", "RC"]
    ]

def command_acl():
    acl_tlist_all = call(["acl_tlist", "-n"])
    all_temp = call(["net_temp", "list_temp_nr"])
    cmd = [
        ["acl_tlist", "-br", "FD", ":DEFAULT:", "/"],
        ["acl_tlist", "-b", "DEV", ":DEFAULT:"],
        ["acl_tlist", "-Db"],
        ["acl_tlist", "-br", "IPC", ":DEFAULT:"],
        ["acl_tlist", "-br", "SCD", ":DEFAULT:", acl_tlist_all],
        ["acl_tlist", "-ab"],
        ["acl_tlist", "-br", "PROCESS", ":DEFAULT:"],
        ["acl_tlist", "-br", "NETDEV", ":DEFAULT:"],
        ["acl_tlist", "-br", "NETTEMP_NT", ":DEFAULT:", all_temp],
        ["acl_tlist", "-br", "NETTEMP", all_temp],
        ["acl_tlist", "-br", "NETOBJ", ":DEFAULT:"],
        ["acl_mask", "-br", "FD", "/"],
        ["acl_mask", "-Db"],
        ["acl_mask", "-ab"],
        ["acl_mask", "-b", "SCD", acl_tlist_all]
        ["acl_group", "-gb", "list_groups"]
    ]
    process1 = Popen(["acl_group", "-gs", "list_groups"], stdout=PIPE, 
            stderr=PIPE)
    process2 = Popen(["cut", "-f", "1", "-d", " "], stdin=process1.stdout, 
            stdout=PIPE, stderr=PIPE)
    groups, error = process2.communictate()
    if groups != "":
        for group in groups:
            cmd.append(["acl_group", "-b", "get_group_members", group])
    return cmd

def command_general():
    cmd = []
    for module, status in rsbac.Rsbac().get_modules()["Module"].items():
        if module in BACKUP_MODULES_EXCLUDE:
            continue
        if status == "on":
            cmd.append(["attr_back_dev", "-b"])
    return cmd

def command_um():
    return [
        ["rsbac_groupshow", "-S", "all", "-b", "-p", "-a"],
        ["rsbac_usershow", "-S", "all", "-b", "-p", "-a"]
    ]
    
def command_res():
    #cmd = [attr_get_user RES 4294967292 res_min]
    cmd = [
        ["attr_back_fd", "-r", "-M", "RES", "/"],
        ["attr_back_user", "-a", "-M", "RES"]
    ]
    return cmd

def command_pax():
    cmd = [
        ["attr_back_fd", "-r", "-M", "PAX", "/"],
        ["attr_back_user", "-a", "-M", "PAX"]
    ]
    return cmd

def command_gen():
    cmd = [
        ["attr_back_fd", "-r", "-M", "GEN", "/"],
        ["attr_back_user", "-a", "-M", ""]
    ]
    return cmd

def get_directories(directory="/"):
    directories = os.listdir(directory)
    #for i i

def command(module, directory=["/"]):
    cmd = [
        ["attr_back_fd", "-r", "-M", module, directory],
        ["attr_back_user", "-a", "-M", module]
    ]
    return cmd


class Backup(object):
    """Backup RSABAC attribute modules based."""

    def __init__(self):
        self.args = {}
        self.shm_dir = None

    def set_args(self, args):
        self.args = args

    def set_log_level(self, log_level):
        log.setLevel(log_level)

    def get_log_level(self):
        return log.getEffectiveLevel()

    def modules_to_backup(self):
        """Get available modules and extend the module list."""
        modules = BACKUP_MODULES
        try:
            for module, status in rsbac.Rsbac().get_modules()["Module"].items():
                if status == "on" and module not in BACKUP_MODULES_EXCLUDE:
                    modules.append(module.lower())
            if os.path.exists(os.path.join(RSBAC_PROC_INFO_DIR, "stats_um")):
                modules.append("um")
        except Exception as error:
             log.error(error)
        return modules

    def execute(self, backup_dir, module):
        """Execute the module command and write to different files the result.
            The name are build: 
                backup_timestamp.sh
                command_timestamp.sh
                error_timestamp.txt
        """
        # build function name and call the function to get the shell commands
        cmd = globals()["_".join(["command", module])]()

        # build names for writing outupt 
        backup = os.path.join(backup_dir, 
                "backup_" + TIMESTAMP + ".sh")
        error = os.path.join(backup_dir, 
                "error_" + TIMESTAMP + ".txt")
        command = os.path.join(backup_dir, 
                "command_" + TIMESTAMP + ".sh")
       
        try:
            with open(backup, "w", buffering=BUFFER_SIZE) as backup_fd, \
                    open(error, "w", buffering=BUFFER_SIZE) as error_fd, \
                    open(command, "w", buffering=BUFFER_SIZE) as command_fd:
                for i in cmd:
                    command_fd.write(" ".join([str(x) for x in i]) + "\n")
                    process = Popen(i, bufsize=BUFFER_SIZE, stdin=PIPE, 
                            stdout=backup_fd, stderr=error_fd)
                    process.wait()
        except IOError as error:
            log.error(error)
   
    def check_shm(self):
        if not os.path.isdir("/dev/shm"):
            log.error("/dev/shm is not mounted")
            return False
        user = os.environ.get("USER")
        if len(user) == 0:
            log.error("could not found a valid user")
            return False
        try:
            path = os.path.join("/dev/shm", user)
            if not os.path.isdir(path):
                log.info("create %s" % path)
                os.mkdir(path)
            self.shm_dir = path
            return True
        except OSError as error:
            log.error(error)
            return False

    def get_backup_dir(self):
        backup_dir = os.path.join(self.args.get("backup_directory", 
                BACKUP_DIRECTORY), BACKUP_DIRECTORY_TODAY)
        try:
            if not os.path.isdir(backup_dir):
                os.makedirs(backup_dir)
            log.info("backup_dir is %s" % backup_dir)
            return backup_dir
        except OSError as error:
            log.error(error)
            return False

    def copy_files(self, src_dir, dst_dir):
        files = os.listdir(src_dir)
        if len(files) == 0:
            log.error("no files found to copy")
            return False
        for file in files:
            try:
                src_file = os.path.join(src_dir, file)
                dst_file = os.path.join(dst_dir, file)
                log.info("copy %s to backup_dir" % src_file)
                shutil.copy(src_file, dst_file)
            except OSError as error:
                log.error(error)
                return False
        return True

    def run(self):
        backup_dir = self.get_backup_dir()
        if not self.check_shm():
            log.error("do: mount -t tmpfs none /dev/shm")
            return False
        with tempfile.TemporaryDirectory(dir=self.shm_dir) as tmp_dir:
            log.info("create %s" % tmp_dir)
            os.chmod(tmp_dir, 0o700)
            for module in self.modules_to_backup():
                if self.args.get("full", False) is False:
                    if module in self.args:
                        continue
                try:
                    log.info("starting backup: %s" % module)
                    module_dir = os.path.join(tmp_dir, module)
                    if not os.path.isdir(module_dir):
                        log.info("create %s" % module_dir)
                        os.mkdir(module_dir)
                    backup_module_dir = os.path.join(backup_dir, module)
                    if not os.path.isdir(backup_module_dir):
                        log.info("create %s" % backup_module_dir)
                        os.mkdir(backup_module_dir)
                    self.execute(module_dir, module)
                    self.copy_files(module_dir, backup_module_dir)
                except KeyError as error:
                    log.error(error)
                    log.info("Not implemented: %s()" % module)
                except OSError as error:
                    log.info(error)
        return True


class BackupNg(object):

    def __init__(self):
        self.args = {}
        self.shm_dir = None
        self.files_dirs = []
        self.dirs = ["/bin", "/etc/", "/lib", "/lib64", "/sbin", "/usr", "/opt"]

    def set_args(self, args):
        self.args = args

    def set_log_level(self, log_level):
        log.setLevel(log_level)

    def get_log_level(self):
        return log.getEffectiveLevel()

    def modules_to_backup(self):
        """Get available modules and extend the module list."""
        modules = BACKUP_MODULES
        try:
            for module, status in rsbac.Rsbac().get_modules()["Module"].items():
                if status == "on" and module not in BACKUP_MODULES_EXCLUDE:
                    modules.append(module.lower())
            if os.path.exists(os.path.join(RSBAC_PROC_INFO_DIR, "stats_um")):
                modules.append("um")
        except Exception as error:
             log.error(error)
        return modules

    def run(self):
        from queue import Queue
        from threading import Thread

        def do_it():
            pass

        def worker():
            item = q.get()
            do_work(item)
            q.task_done()

        q = Queue()
        num_worker_threads = 3
        for i in range(num_worker_threads):
                t = Thread(target=worker)
                t.daemon = True
                t.start()

        for item in source():
                q.put(item)

        q.join()       # block until all tasks are done

        if not self.get_files_dirs():
            return False
        for module in self.modules_to_backup():
            print(module)

        print(len(self.files_dirs))
        

    def get_files_dirs(self):
        try:
            for i in self.dirs:
                for j in os.listdir(i):
                    self.files_dirs.append(os.path.join(i, j))
            return True
        except OSError as error:
            log.error(error)



if __name__ == "__main__":
    # 0.009 sec to collect all give files_dirs 
    start_time = datetime.datetime.now().timestamp()
    b = BackupNg()
    b.set_log_level(logging.DEBUG)
    b.run()
    end_time = datetime.datetime.now().timestamp()
    result_time = end_time - start_time
    print(result_time)
    #print(str("%0.10f" % result_time))
